#
# PURPOSE:
# Create a kml file containing randomly distributed polygons for further mapping of rock glaciers or any other random sampling in
# Google Earth. The way this function is set up using set.seed() the sample X will always be at position X with the same name.
# Therefore the parameters sample.start and sample.end have to be defined to get a defined subset and accordingly a number of polygons.
#
# Every polygon will get a unique name consisting of two capital letters and three numbers. The first polygon gets the name AA001.
#
#
# INPUTS:
#	Shapefile with one [no islands!] polygon delineating the area of interest
#
# PARAMETERS:
#   in.file.shp:  shape file as described for INPUTS
#   out.file.kml: name and location of kml output file
#   sample.start: Which location and name to start with
#   sample.end:   Which location and name to end with -> Difference between sample.start and sample.end is equal the number of polygons
#                 in the output file!
#   sample.length:Longitudinal length of sample polygons in decimal degrees 
#
# OUTPUT:
# kml file with sample.start of polygons with a latitudinal length of sample.length
#
# ERRORS: Error in sample.int(length(x), size, replace, prob) : cannot take a sample larger than the population when 'replace = FALSE'
# ISSUE: The number of output polygons exceeds the number of possible polygons within the boundary defined by in.file.shp.
#        Either decrease number of polygons (sample.start and sample.end closer together) or decrease sample.length (size of polygons)
#
# EXAMPLE:
# [1] sample.start = 0, sample.end=50 => creates 50 randomly distribued polygons starting at AA001
# [2] sample.start = 50, sample.end=100 => creates 50 randomly distribued polygons starting at AA050
#
# FURTHER INFORMATION:
# Schmid, M.-O., Baral, P., Gruber, S., Shahi, S., Shrestha, T., Stumm, D. and Wester, P. (2014) 
# Assessment of permafrost distribution maps in the Hindu Kush-Himalayan region using rock glaciers
# mapped in Google Earth, Cryosph. Discuss., 8(5), 5293-5319, doi:10.5194/tcd-8-5293-2014
#
# AUTHOR: 
# schmid Marc-Olivier [marcolivier.schmid@gmail.com]
#
# CITATION:
# Schmid, M.-O., Baral, P., Gruber, S., Shahi, S., Shrestha, T., Stumm, D., and Wester, P.: 
# Assessment of permafrost distribution maps in the Hindu Kush Himalayan region using rock glaciers mapped in Google Earth, 
# The Cryosphere, 9, 2089-2099, doi:10.5194/tc-9-2089-2015, 2015.
#
###############################################################################

###############################################################################
###############################################################################
# TEST function
#
# rm(list = ls())
# 
# f.RandomPolygon(in.file.shp ="D:/PF Map/R Workspace/Outline/AA/GR.shp",
#               out.file.kml = "D:/PF Map/R Workspace/kml/TESTING/TEST4.kml",
#               sample.start = 80,
#               sample.end=200,
#               sample.length = 0.05)
###############################################################################
###############################################################################

f.RandomPolygon  <- function(in.file.shp,out.file.kml,sample.start,sample.end,sample.length){


library(maptools)    	# for kml
library(ggplot2)  		# for fortify

###############################################################################
###############################################################################
###############################################################################
### PART I - Create sample polygon coordinates

### Read in boundaries
bound  <-  readShapePoly(in.file.shp)

### Get cooridinates (ggplot2 needed)
bound.matrix <-  fortify(bound)

### Create all Lat/Lon values
lat.max  <-  round(max(bound.matrix$lat)+sample.length,1)
lat.min  <-  round(min(bound.matrix$lat)-sample.length,1)

lon.max  <-  round(max(bound.matrix$long)+sample.length,1)
lon.min  <-  round(min(bound.matrix$long)-sample.length,1)

lat	<-	seq(lat.min,lat.max,sample.length)
lon <-	seq(lon.max,lon.min,-sample.length)


### Correction for latitude
deg2rad = function(deg) {
  return((pi * deg) / 180)
}

### Create longitude dataframe
londat <-as.data.frame(matrix(ncol=length(lon),nrow=length(lat)))


n  <-  1

for (i in 1:length(lat)){
  
  sample.length.lat  <-  sample.length/cos(deg2rad(lat[i]))
  
  londat[n,]  <-  seq(lon.max,by=-sample.length.lat,length.out=length(lon))
  
  n  <-  n+1
}

### Create latitude dataframe
latdat <-as.data.frame(matrix(ncol=length(lon),nrow=length(lat)))

n  <-  1

for (i in 1:length(latdat$V1))
{
  latdat[i,]<-lat[n]
  
  n  <-  n+1
}

########################################################################
### Create all polygons (4points, 5th closes the ring)

x  <-  rep(NA,5*nrow(latdat)*ncol(latdat))
y  <-  x

i <- 1

### Create progress bar for outer loop
########################################################################
pb <- winProgressBar(title = "progress bar create polygons", min = 0, max = nrow(latdat), width = 300)
########################################################################


for (n in 2:nrow(latdat))
{
  
  for (nn in 1:(ncol(latdat)-1))
    
  {
    ### Check if 'first' point of polygon is within the boundaries
    if(point.in.polygon(latdat[n,nn],londat[n,nn],bound.matrix$lat,bound.matrix$long))
    {
      
      x[i]	<-latdat[n,nn]
      x[i+1]	<-x[i]
      x[i+2]	<-latdat[n-1,nn]
      x[i+3]	<-x[i+2]
      x[i+4]	<-x[i]
      
      y[i]	<-londat[n,nn]
      y[i+1]	<-londat[n,nn+1]
      y[i+2]	<-y[i+1]
      y[i+3]	<-y[i]
      y[i+4]	<-y[i]
      

{
  i <- i+5
}
    }
  }
  
  ### ProgressBar
  setWinProgressBar(pb, n)
  
}

### Close progress bar for outer loop
########################################################################
close(pb)
########################################################################

data		<-na.omit(cbind(y,x))

########################################################################
########################################################################
### PART II - Create kml file

### Create list with "starting" points for each polygon (list length = nummber of polygons)
all.poly  <-  seq(1,length(data[,1]),5)

### Create random sample accoriding to defined sample size

set.seed(1)   #Set seed -> allways same 'random' samples

sub.poly  <-  sample(all.poly, sample.end, replace = FALSE)
sub.poly  <-  sub.poly[sample.start:sample.end]

### Number of samples
sample.nr  <-  sample.end-sample.start

### Create SpatialPolygonsDataFrame
i<-1
spdf.list = vector("list",sample.nr)

for (n in sub.poly){
  
  dat	<- data[n:(n+4),]
  
  
  sps = SpatialPolygons(list(Polygons(list(Polygon(dat[,1:2])),1)))
  proj4string(sps) = CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
  
  da = data.frame(z=i)
  
  spdf.list[i] = SpatialPolygonsDataFrame(sps,da)
  
  
  i<-i+1
  
}


########################################################################
### Creates names for polygons

### Trick to create names
"&" <- function(...) UseMethod("&")
"&.default" <- .Primitive("&")
"&.character" <- function(...) paste(...,sep="")

names.list  =  vector("list",sample.nr)
seq.n  =  seq(1:100)

n  <- 1

for(p in 1:26){
  
  let1  <-	LETTERS[seq( from = 1, to = 26 )][p]
  
  
  for (pp in 1:26){
    
    let2  <-	LETTERS[seq( from = 1, to = 26 )][pp]
    
    for (ppp in seq.n){
      
      num1  <-sprintf("%03d", seq.n)[ppp]	
      
      names.list[n]  <- let1&let2&num1
      n  <-  n+1
      
      
      if ( n>=sample.end+1) break
    }
    
    if ( n>=sample.end+1) break
  }
  
  if ( n>=sample.end+1) break
}

names.list  <-  names.list[sample.start:sample.end]

########################################################################
### Creates a KML file 
suppressWarnings(kmlPolygons(spdf.list,out.file.kml,lwd=3,name=names.list))

### Give information concerning number of samples
cat("\nNumber of possible polygons:",nrow(data/5))
cat("\n\nNumber of polygons in output:",sample.nr)
cat("\n\nFrom polygons",sample.start,"to",sample.end)

}
